# CellTracking.py
# By MW, Jun 2013
# GPLv3+
#
# Class managing a cell
import logging, cv2, imp, numpy, sys
sys.path.append('./bin/Features/')
import Features
TimeTrack = imp.load_source("TimeTrack", './bin/Movies/Cells/TimeTrack.py')

class Cell :
    def __init__(self, movie, name, begin, end) :
        self.movie = movie
        self.name = name
        self.begin = begin
        self.end = end
        # Initialize timetracks (empty)
        self.cell_pol = TimeTrack.TimeTrack() # The cell inner polygon
        self.cell_ref = TimeTrack.TimeTrack() # The cell reference polygon
        #self.begin = 0
        #self.end = movie.get_frame_nb()
        self.center_pol = 'mean'
        self.center_ref = None
        self.handle_dist = 8 # The tolerance distance of the cell
        self.features = Features.Features()
    
    def get_features(self) :
        return self.features
    def reset_polygon(self, pol, t) :
        self.cell_pol = TimeTrack.TimeTrack()
        self.cell_pol.add_timepoint(t, pol)
        self.cell_ref = TimeTrack.TimeTrack()
        self.generate_ref()
    def set_name(self, n) :
        self.name = str(n)
    def get_name(self) :
        return self.name
    def generate_ref(self) :
        pass
    def is_point_in_cell(self, pt, t) :
        pol = self.cell_pol.get_polygon(t)
        if cv2.pointPolygonTest(numpy.array([pol], dtype=numpy.int32), pt, False) == 1.0 :
            return True
        else :
            return False
    def is_point_in_ref(self, pt, t) :
        #logging.error("Is point in ref not implemented yet, returning False")
        pol = self.cell_ref.get_polygon(t)
        if cv2.pointPolygonTest(numpy.array([pol], dtype=numpy.int32), pt, False)==1.0 :
            return True
        else :
            return False
        #return False
    def is_point_on_cell_handle(self, pt, t) :
        """Tests if the point lies within a given distance of a handle (an edge, and maybe a center"""
        return self.point_on_handle(pt, t, self.center_pol, self.cell_pol)
    def is_point_on_ref_handle(self, pt, t) :
        return self.point_on_handle(pt, t, self.center_ref, self.cell_ref)

    def point_on_handle(self, tup, t, center_type, timetr) :
        """Internal function returning the selected edge of the polygon (if 
        any), the center or None
        This function was coded with my feet, it is a total mess"""
        #offset = numpy.array([0,0])
        pol = timetr.get_polygon(t)
        selected = None
        for (i,p) in enumerate(pol) :
            if self.dist(p, tup, self.handle_dist) :
                selected = i
        #if center_type == 'mean_translated' :
        #    offset[0] = 10
        #    print 'reference'
        if center_type == 'mean' or center_type=='mean_translated':
            center = numpy.array(timetr.get_center(t))#+offset
            if self.dist(tup, center, self.handle_dist) :
                selected = 'center'
        return selected
    
    def dist(self, pt1, pt2, rad) :
        return ((pt1[0]-pt2[0])**2+(pt1[1]-pt2[1])**2 < rad**2)
    def init_ref(self, refsize) :
        """Function that initialize a reference at a given distance of the
        polygon"""
        for i in self.cell_pol.times :
            self.update_ref(i, refsize)
            #ref = self.compute_ref(self.cell_pol.get_polygon(i), refsize)
            #self.cell_ref.add_timepoint(i, ref)
    def update_ref(self, i, refsize) :
        ref = self.compute_ref(self.cell_pol.get_polygon(i), float(refsize))
        self.cell_ref.add_timepoint(i, ref)
    def set_begin(self, i) :
        self.cell_pol.set_begin(i)
        self.cell_ref.set_begin(i)
        self.begin = i
    def set_end(self, i) :
        self.cell_pol.set_end(i)
        self.cell_ref.set_end(i)
        self.end = i
    def get_bounds(self) :
        return (self.begin, self.end)
        
    def compute_ref(self, pol, refsize) :
        pol = numpy.array(pol)
        tr = pol.mean(axis=0)
        pol_norm = (((pol-tr)**2).sum(axis=1))**0.5
        pol_norm = numpy.vstack((pol_norm, pol_norm)).transpose()
        return (pol-tr)*(pol_norm+refsize)/pol_norm+tr
    def delete(self) :
        """Function called to delete the cell"""
        logging.warning("Delete cell yet to be implemented")

    def export_as_tiff(self, channel) :
        """Function exports the cell as a tiff series
        
        Think about managing the number of channels"""
        bounds = self.get_bounds()
        for i in range(bounds[0], bounds[1]) : 
            # Cell
            c=self.get_cell(i, channel, show_cel=True)
            cv2.imwrite('./exports/cell'+str(i)+".tiff", c.filled(0))
            # Reference
            r=self.get_cell(i, channel, show_ref=True)                        
            cv2.imwrite('./exports/ref'+str(i)+".tiff", r.filled(0))

    def export_as_csv(self, channel) :
        """Function exports the cell as a tiff series
        
        Think about managing the number of channels"""
        bounds = self.get_bounds()
        dt=[('frame', float),
            ('ref_area', float),
            ('ref_avg', float),
            ('ref_sd', float),
            ('cell_area', float),
            ('cell_avg', float),
            ('cell_sd', float)]
        fmt = "%10.5f"
        out = numpy.zeros((bounds[1]-bounds[0], 7), dtype=dt)
        fil = open('./exports/cell.csv', 'w')
        for n in dt :
            fil.write(n[0]+'\t')
        fil.write('\n')
        for (i,frame) in enumerate(range(bounds[0], bounds[1])) : 
            # Cell/ref
            c=self.get_cell(frame, channel, show_cel=True)
            r=self.get_cell(frame, channel, show_ref=True)
            
            #print frame, type(frame), out.shape, out.dtype.names
            out['frame'][i] = frame
            out['ref_area'][i] = r.count()
            out['ref_avg'][i] = r.mean()
            out['ref_sd'][i] = r.std()
            out['cell_area'][i] = c.count()
            out['cell_avg'][i] = c.mean()
            out['cell_sd'][i] = c.std()
            
            fil.write(str(frame)+'\t'
                      +str(r.count())+'\t'
                    +str(r.mean())+'\t'
                    +str(r.std())+'\t'
                    +str(c.count())+'\t'
                    +str(c.mean())+'\t'
                    +str(c.std())+'\n')
            print 'frame:', i

        #numpy.savetxt('./exports/cell.csv', out, fmt=fmt)#, delimiter='\t')
        fil.close()

    def get_cell(self, i, channel, show_ref=False, show_cel=False) :
        cel = self.cell_pol.get_polygon(i)
        ref = self.cell_ref.get_polygon(i)
        
        f=self.movie.get_frame(i, channel, rgb=False)
        mask = numpy.ones_like(f)

        cv2.fillPoly(mask, [numpy.array(ref, dtype=numpy.int32)], color=(1-show_ref))
        cv2.fillPoly(mask, [numpy.array(cel, dtype=numpy.int32)], color=(1-show_cel))
        
        mask_i = numpy.ma.array(f, mask=numpy.bool_(mask), fill_value=0)
        return mask_i

    def get_save_dict(self) :
        dic = {}
        dic['name'] = {'descr' : 'name of the cell', 'value' : self.name}
        dic['begin'] = {'descr' : 'beginning frame of the cell', 'value' : self.begin} 
        dic['end'] = {'descr' : 'ending frame of the cell', 'value' : self.end} 
        dic['cell_pol'] = {'descr' : 'Timetrack instance describing the borders of the cell', 'value' : self.cell_pol.get_save_dict()}
        dic['cell_ref'] = {'descr' : 'Timetrack instance describing the borders of the reference polygon', 'value' : self.cell_ref.get_save_dict()}
        dic['center_pol'] = {'descr' : 'type of center of the cell.', 'value' : self.center_pol}
        dic['center_ref'] = {'descr' : '?', 'value' : self.center_ref}
        dic['handle_dist'] = {'descr' : 'Distance from the center of the handle to consider the click as a try to move the handle', 'value' : self.handle_dist}
        dic['features'] = {'descr' : 'The features object associated with the cell', 'value' : self.features.get_save_dict()}
        return {'descr' : 'a cell object', 'value' : dic}

    def load_save_dict(self, dic) :
        self.set_name(dic['value']['name']['value'])
        self.handle_dist = dic['value']['handle_dist']['value']
        self.center_pol = dic['value']['center_pol']['value']
        self.center_ref = dic['value']['center_ref']['value']
        self.cell_pol.load_save_dict(dic['value']['cell_pol']['value'])
        self.cell_ref.load_save_dict(dic['value']['cell_ref']['value'])
        # After cell_pol and cell_ref
        self.set_begin(dic['value']['begin']['value'])
        self.set_end(dic['value']['end']['value'])
        self.features.load_save_dict(dic['value']['features']['value'])
